<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

/**
 * Class to manage database-based sessions.
 * The configuration (table, fields, and so on) is in the
 * base configuration (see mymvc.php)
 */
class myMVC_SessionDB extends myMVC_BaseDB
{
  /**
   * Constructor.
   */
  public function __construct()
  {
    parent::__construct('', array(), array());
  }

  /**
   * Get session data from database
   * 
   * @param string $sessionId               The SessionID
   * @return array                          The session data
   */
  public function getSessionDataFromDB($sessionId)
  {
    global $config;

    $sessData = '';
    $cfg = $config['SESSION_DB'];
    $query = sprintf("SELECT %s FROM %s WHERE %s = '%s'",
                    $cfg['field_data'], $cfg['table'], $cfg['field_sid'], $sessionId);
    if(($ret = $this->findBySQLQuery($query)) !== NULL)
    {
      if($this->engine->getRowsNum($ret) != 0)
      {
        $row = $this->engine->getValueArrayFromResult($ret);
        $sessData = $row[$cfg['field_data']];
        $this->engine->freeResult($ret);
      }
    }
    else
      return NULL;
    return unserialize($sessData);
  }

  /**
   * Save session data into the database
   * 
   * @param string $sessionId               The SessionID
   * @param array $sessionData              The session data
   */
  public function saveSessionDataIntoDB($sessionId, $sessionData)
  {
    global $config;

    $hasToInsert = true;
    $sessData = array();
    $cfg = $config['SESSION_DB'];
    $query = sprintf("SELECT %s FROM %s WHERE %s = '%s'",
                    $cfg['field_data'], $cfg['table'], $cfg['field_sid'], $sessionId);
    if(($ret = $this->findBySQLQuery($query)) !== NULL)
    {
      if($this->engine->getRowsNum($ret) != 0)
      {
        $hasToInsert = false;
      }
      $this->engine->freeResult($ret);
    }

    if($hasToInsert)
      $query = sprintf("INSERT INTO %s (%s, %s, %s) VALUES('%s', '%s', NOW())",
                      $cfg['table'], $cfg['field_sid'], $cfg['field_data'], $cfg['field_last_used'],
                      $sessionId, serialize($sessionData));
    else
      $query = sprintf("UPDATE %s SET %s = '%s', %s = NOW() WHERE %s = '%s'",
                      $cfg['table'], $cfg['field_data'], serialize($sessionData),
                      $cfg['field_last_used'], $cfg['field_sid'], $sessionId);
    $this->engine->execute($query);
  }

  /**
   * Delete old session from database
   */
  public function deleteOldSessions()
  {
    global $config;

    $cfg = $config['SESSION_DB'];
    $query = sprintf("DELETE FROM %s WHERE %s <= NOW() - %s",
                    $cfg['table'], $cfg['field_last_used'], $config['SESSION_LIFETIME']);
    $this->engine->execute($query);
  }
}
?>
