<?php
class AutoresponderAction extends GUIAction
{
  public function __construct($request, $session)
  {
    parent::__construct($request, $session);
    $this->setCheckFunction('POST', 'validatePost');
  }

  /**
   * Returns the CSS class of this action
   * 
   * @return string                     The CSS class
   */
  public static function getCSSClass()
  {
    return 'general';
  }

  /**
   * Returns the token used to identify the current Action
   * 
   * @return string                 The token
   */
  public static function getActionToken()
  {
    return 'autoresponder';
  }

  /**
   * Returns whether this action should appear in the menu (created by controller)
   * 
   * @return boolean                true if the action should appear in menu
   */
  public static function isMenuAction()
  {
    return true;
  }

  /**
   * Returns the position of this action in the menu (created by controller)
   * Only necessary if isMenuAction() returns true
   * 
   * @return int                        The position in the menu (lower is "upper")
   */
  public static function getActionMenuPosition()
  {
    return 15;
  }

  /**
   * Returns the string to be displayed in the menu to identify the current action.
   * Only necessary if isMenuAction() returns true
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuName()
  {
    return _("Autoresponder");
  }

  /**
   * Gibt den Name des Bildes für dieses Menüpunkt zurücl
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuImage()
  {
    return '';
  }

  /**
   * Returns whether the current user is allowed to call the current Action
   * 
   * @return boolean                true if the user may call this Action, false otherwise
   */
  public static function hasUserRightForAction()
  {
    $user = self::getLoggedUser();
    if($user !== NULL && ($user->getIsAdmin() != 'f' || $user->getCanSetAutoresponder() == 't'))
      return true;
    return false;
  }
  
  /**
   * Validate the data from user, coming as POST
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validatePost()
  {
    $user = self::getLoggedUser();
    $users = array('__SELF__');
    foreach($user->getUsersForAdmin() as $u)
      array_push($users, $u->getAccountname());
    if($this->request->getSubmitParameter('action') == 'chgAdminUser')
    {
      $definitions = array(
          'adminUser'           => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $users),
      );
    }
    elseif($this->request->getSubmitParameter('save') != '')
    {
      $definitions = array(
        'hasAutoresponder'      => array('type' => myMVC_BaseAction::$VALIDATE_BOOLEAN, 'empty' => false),
        'autoresponderSubject'  => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'maxlen' => 255, 'empty' => ($this->request->getSubmitParameter('hasAutoresponder') == 'f')),
        'autoresponderText'     => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'maxlen' => 65535, 'empty' => ($this->request->getSubmitParameter('hasAutoresponder') == 'f')),
      );
      if($user->getIsAdmin() != 'f')
      {
        $users = array('__SELF__');
        foreach($user->getUsersForAdmin() as $u)
          array_push($users, $u->getAccountname());
        $definitions['adminUser'] =  array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $users);
      }
    }
    $names = array(
        'hasAutoresponder'      => _('Autoresponder'),
        'autoresponderSubject'  => _('Autoresponder-Betreff'),
        'autoresponderText'     => _('Autoresponder-Text'),
        'adminUser'             => _('Administrierter Nutzer'),
    );
    $errorMsgs = array(
        'hasAutoresponder'      => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTBOOL       => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'autoresponderSubject'  => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_MAXLEN        => sprintf(_("%s ist zu lang"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTSTRING     => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'autoresponderText'     => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_MAXLEN        => sprintf(_("%s ist zu lang"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTSTRING     => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'adminUser'             => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_RANGE         => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTSTRING     => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
    );
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Perform the Action
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function perform()
  {
    global $config;

    $response = new myMVC_HttpResponse();

    $response->setNext('AutoresponderView');
    $this->session->useNamespace('autoresponder');
    $user = self::getLoggedUser();

    if($this->request->getRequestMethod() == 'GET')
    {
      $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
      $this->saveParameterForNextPage('account', $acc);
      $this->saveParameterForNextPage('hasAutoresponder', $acc->getHasAutoresponder());
      $this->saveParameterForNextPage('autoresponderSubject', $acc->getAutoresponderSubject());
      $this->saveParameterForNextPage('autoresponderText', $acc->getAutoresponderText());
    }
    else
    {
      if($this->request->getSubmitParameter('action') == 'chgAdminUser')
      {
        if($this->request->getParameter('adminUser') != '__SELF__')
          $acc = Account::findByPK(array('accountname' => $this->request->getParameter('adminUser')));
        else
          $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        $this->saveParameterForNextPage('account', $acc);
        $this->saveParameterForNextPage('hasAutoresponder', $acc->getHasAutoresponder());
        $this->saveParameterForNextPage('autoresponderSubject', $acc->getAutoresponderSubject());
        $this->saveParameterForNextPage('autoresponderText', $acc->getAutoresponderText());
        $this->saveParameterForNextPage('adminUser', $this->request->getParameter('adminUser'));
      }
      elseif($this->request->getSubmitParameter('save') != '')
      {
        if($user->getIsAdmin() != 'f' && $this->request->getParameter('adminUser') != '__SELF__')
          $acc = Account::findByPK(array('accountname' => $this->request->getParameter('adminUser')));
        else
          $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        $acc->setHasAutoresponder($this->request->getParameter('hasAutoresponder'));
        $acc->setAutoresponderSubject($this->request->getParameter('autoresponderSubject'));
        $acc->setAutoresponderText($this->request->getParameter('autoresponderText'));
        $acc->save();
        if($user->getIsAdmin() != 'f')
          $this->saveParameterForNextPage('adminUser', $this->request->getParameter('adminUser'));
        else
          $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        $this->saveParameterForNextPage('account', $acc);
        $this->saveParameterForNextPage('hasAutoresponder', $acc->getHasAutoresponder());
        $this->saveParameterForNextPage('autoresponderSubject', $acc->getAutoresponderSubject());
        $this->saveParameterForNextPage('autoresponderText', $acc->getAutoresponderText());
        $this->saveParameterForNextPage('popupMessage', _('Die Einstellungen wurden erfolgreich gespeichert'));
      }
    }

    return $response;
  }

  /**
   * Called if user data are not valid (check with validate)
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function manageInvalidData()
  {
    $response = new myMVC_HttpResponse();
    $response->setNext('AutoresponderView');
    $this->saveParameterForNextPage('error', true);

    return $response;
  }
}
?>
