<?php
class ManageFilterAction extends GUIAction
{
  public function __construct($request, $session)
  {
    parent::__construct($request, $session);
    $this->setCheckFunction('GET', 'validateGet');
    $this->setCheckFunction('POST', 'validatePost');
  }

  /**
   * Returns the CSS class of this action
   * 
   * @return string                     The CSS class
   */
  public static function getCSSClass()
  {
    return 'general';
  }

  /**
   * Returns the token used to identify the current Action
   * 
   * @return string                 The token
   */
  public static function getActionToken()
  {
    return 'manageFilter';
  }

  /**
   * Returns whether this action should appear in the menu (created by controller)
   * 
   * @return boolean                true if the action should appear in menu
   */
  public static function isMenuAction()
  {
    return true;
  }

  /**
   * Returns the position of this action in the menu (created by controller)
   * Only necessary if isMenuAction() returns true
   * 
   * @return int                        The position in the menu (lower is "upper")
   */
  public static function getActionMenuPosition()
  {
    return 50;
  }

  /**
   * Returns the string to be displayed in the menu to identify the current action.
   * Only necessary if isMenuAction() returns true
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuName()
  {
    return _("Filter");
  }

  /**
   * Gibt den Name des Bildes für dieses Menüpunkt zurücl
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuImage()
  {
    return '';
  }

  /**
   * Returns whether the current user is allowed to call the current Action
   * 
   * @return boolean                true if the user may call this Action, false otherwise
   */
  public static function hasUserRightForAction()
  {
    $user = self::getLoggedUser();
    if($user !== NULL && ($user->getIsAdmin() != 'f' || $user->getCanUseFilters() == 't'))
      return true;
    return false;
  }

  /**
   * Validate the data from user, coming as GET
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validateGet()
  {
    $this->session->useNamespace('manageFilter');
    $filters = $this->session->get('filters');
    if($this->request->getSubmitParameter('delete') != '')
    {
      $definitions = array(
          'delete' => array('type' => myMVC_BaseAction::$VALIDATE_NUMBER, 'empty' => false, 'min' => 0, 'max' => count($filters)),
      );
    }
    elseif($this->request->getSubmitParameter('edit') != '')
    {
      $definitions = array(
          'edit'   => array('type' => myMVC_BaseAction::$VALIDATE_NUMBER, 'empty' => false, 'min' => 0, 'max' => count($filters)),
      );
    }
    $names = array(
          'delete' => _('Filter zu l&ouml;schen'),
          'edit'   => _('Filter zu &auml;ndern'),
    );
    $errorMsgs = array(
          'delete' => array(myMVC_BaseAction::$ERROR_MIN       => _('Der Filter existiert nicht'),
                            myMVC_BaseAction::$ERROR_MAX       => _('Der Filter existiert nicht'),
                            myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                            myMVC_BaseAction::$ERROR_NOTNUMBER => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
          'edit'   => array(myMVC_BaseAction::$ERROR_MIN       => _('Der Filter existiert nicht'),
                            myMVC_BaseAction::$ERROR_MAX       => _('Der Filter existiert nicht'),
                            myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                            myMVC_BaseAction::$ERROR_NOTNUMBER => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
    );

    if(!is_array($definitions))
      return true;
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Validate the data from user, coming as POST
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validatePost()
  {
    $this->session->useNamespace('manageFilter');
    $definitions = $names = $errorMsgs = array();
    $t = array();
    foreach(Account::$filtersTests as $f)
      array_push($t, $f);
    $user = self::getLoggedUser();
    $users = array('__SELF__');
    foreach($user->getUsersForAdmin() as $u)
      array_push($users, $u->getAccountname());

    if($this->request->getSubmitParameter('action') == 'chgAdminUser')
    {
      $definitions = array(
          'adminUser' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $users),
      );
      $names = array(
          'adminUser' => _('Administrierter Nutzer'),
      );
      $errorMsgs = array(
          'adminUser' => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                               myMVC_BaseAction::$ERROR_RANGE         => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                               myMVC_BaseAction::$ERROR_NOTSTRING     => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
      );
    }
    elseif($this->request->getSubmitParameter('cancel') != '')
    {
      return true;
    }
    elseif($this->request->getSubmitParameter('addLine') != '')
    {
      for($l = 0; $l < $this->session->get('maxLine'); $l++)
      {
        $definitions['filterTest_'.$l]      = array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $t);
        $definitions['filterCondition_'.$l] = array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => Account::$filtersConditions);
        $definitions['value_'.$l]           = array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'maxlen' => 255);
        $definitions['andOr_'.$l]           = array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => true, 'range' => array('AND', 'OR'));
        $names['filterTest_'.$l]            = _('Test');
        $names['filterCondition_'.$l]       = _('Bedingung');
        $names['value_'.$l]                 = _('Wert');
        $names['andOr_'.$l]                 = _('Verbindung');
        $errorMsgs['filterTest_'.$l]        = array(myMVC_BaseAction::$ERROR_RANGE     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}'));
        $errorMsgs['filterCondition_'.$l]   = array(myMVC_BaseAction::$ERROR_RANGE     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}'));
        $errorMsgs['value_'.$l]             = array(myMVC_BaseAction::$ERROR_MAXLEN    => sprintf(_("%s ist zu lang"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}'));
        $errorMsgs['andOr_'.$l]             = array(myMVC_BaseAction::$ERROR_RANGE     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}'));
      }
    }
    elseif($this->request->getSubmitParameter('delLine') != '')
    {
      $definitions['delLineNo']             = array('type' => myMVC_BaseAction::$VALIDATE_NUMBER, 'empty' => false, 'min' => 0, 'max' => $this->session->get('maxLine'));
      $names['delLineNo']                   = _('Zeile zu l&ouml;schen');
      $errorMsgs['delLineNo']               = array(myMVC_BaseAction::$ERROR_MIN       => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_MAX       => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_NOTNUMBER => sprintf(_("%s ist ung&uuml;ltig"), '{field}'));
    }
    elseif($this->request->getSubmitParameter('addFilter') != '')
    {
      $definitions = array(
          'filterAction'                    => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => array_keys(Account::$filtersActions)),
          'destination'                     => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => ($this->request->getSubmitParameter('filterAction') == 'discard'), 'maxlen' => 255),
          'filterActionFinish'              => array('type' => myMVC_BaseAction::$VALIDATE_BOOLEAN, 'empty' => true),
      );
      $names = array(
          'filterAction'                    => _('Aktion'),
          'destination'                     => _('Ziel'),
          'filterActionFinish'              => _('Ende der Bearbeitung'),
      );
      $errorMsgs = array(
          'filterAction'                    => array(myMVC_BaseAction::$ERROR_RANGE     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                     myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                     myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
          'destination'                     => array(myMVC_BaseAction::$ERROR_MAXLEN    => sprintf(_("%s ist zu lang"), '{field}'),
                                                     myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                     myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
          'filterActionFinish'              => array(myMVC_BaseAction::$ERROR_NOTBOOL   => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
      );
      for($l = 0; $l < $this->session->get('maxLine'); $l++)
      {
        $definitions['filterTest_'.$l]      = array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $t);
        $definitions['filterCondition_'.$l] = array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => Account::$filtersConditions);
        $definitions['value_'.$l]           = array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'maxlen' => 255);
        $definitions['andOr_'.$l]           = array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => true, 'range' => array('AND', 'OR'));
        $names['filterTest_'.$l]            = _('Test');
        $names['filterCondition_'.$l]       = _('Bedingung');
        $names['value_'.$l]                 = _('Wert');
        $names['andOr_'.$l]                 = _('Verbindung');
        $errorMsgs['filterTest_'.$l]        = array(myMVC_BaseAction::$ERROR_RANGE     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}'));
        $errorMsgs['filterCondition_'.$l]   = array(myMVC_BaseAction::$ERROR_RANGE     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}'));
        $errorMsgs['value_'.$l]             = array(myMVC_BaseAction::$ERROR_MAXLEN    => sprintf(_("%s ist zu lang"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}'));
        $errorMsgs['andOr_'.$l]             = array(myMVC_BaseAction::$ERROR_RANGE     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                    myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}'));
      }
    }
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Perform the Action
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function perform()
  {
    global $config;

    $response = new myMVC_HttpResponse();

    $response->setNext('ManageFilterView');
    $this->session->useNamespace('manageFilter');
    $user = self::getLoggedUser();
    if($this->isCalledFromMenu())
    {
      $this->session->emptyNamespace();
      $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
      $this->session->set('usedAccount', $acc);
    }
    else
      $acc = $this->session->get('usedAccount');

    if($this->request->getRequestMethod() == 'GET')
    {
      $this->session->set('edit', NULL);
      if($this->request->getSubmitParameter('delete') != '')
      {
        if($user->getIsAdmin() != 'f')
          $this->saveParameterForNextPage('adminUser', $acc->getAccountname());
        $this->saveParameterForNextPage('account', $acc);

        $filters = $this->session->get('filters');
        $newFilters = array();
        for($l = 0; $l < count($filters); $l++)
          if($l != $this->request->getParameter('delete'))
            array_push($newFilters, $filters[$l]);
        $filterStr = str_replace(' endif', 'endif', join("\n", $newFilters));
        $this->session->set('filters', $newFilters);
        $acc->setFilters($filterStr);
        $acc->save();
        $this->saveParameterForNextPage('popupMessage', _('Der Filter wurde erfolgreich gel&ouml;scht'));
      }
      elseif($this->request->getSubmitParameter('edit') != '')
      {
        if($user->getIsAdmin() != 'f')
          $this->saveParameterForNextPage('adminUser', $acc->getAccountname());
        $this->saveParameterForNextPage('account', $acc);

        $filters = $this->session->get('filters');
        $tokens = explode("\n", $filters[$this->request->getParameter('edit')]);
        $nConditions = 0;
        for($nT = 0; $nT < count($tokens); $nT++)
        {
          $usedWords = 0;
          $token = rtrim(trim($tokens[$nT]));
          if(strtolower($token) == 'and' || strtolower($token) == 'or')
          {
            $filterAndOr[$nConditions] = strtoupper($token);
            $nConditions++;
          }
          if(strtolower($token) == 'seen finish' && $filterAction != 'discard')
            $filterActionFinish = 't';
          if(strtolower($token) == 'then')
          {
            $str = rtrim(trim($tokens[$nT + 1]));
            foreach(array_keys(Account::$filtersActions) as $k)
            {
              list($a, $v) = Account::$filtersActions[$k];
              if(substr($str, 0, strlen($a)) == $a)
              {
                $filterAction = $k;
                $filterActionValue = substr($str, strlen($a) + 1);
              }
            }
            if($filterAction == 'discard')
              $filterActionFinish = 'f';
            if($filterAction == 'save')
              $filterActionValue = str_replace('%FINALDEST%/.', '', $filterActionValue);
          }
          if(substr($token, 0, 1) == '$')
          {
            $ts = explode(' ', $token);
            foreach(array_keys(Account::$filtersTests) as $k)
              if(Account::$filtersTests[$k] == $ts[0])
              {
                $filterTest[$nConditions] = Account::$filtersTests[$k];
                break;
              }
            foreach(Account::$filtersConditions as $k)
            {
              $nw = count(explode(' ', $k));
              $c = '';
              for($l = 0; $l < $nw; $l++)
              {
                if($c != '')
                  $c .= ' ';
                $c .= trim($ts[$l + 1]);
              }
              if($k == $c)
              {
                $usedWords = $nw;
                $filterCondition[$nConditions] = $k;
              }
            }
            $v = '';
            for($l = $usedWords + 1; $l < count($ts); $l++)
            {
              if($v != '')
                $v .= ' ';
              $v .= trim($ts[$l]);
            }
            $v = str_replace('"', '', $v);
            $match[$nConditions] = $v;
          }
        }
        $nConditions++;

        $this->session->set('maxLine', $nConditions);
        $this->session->set('filterTest', $filterTest);
        $this->session->set('filterCondition', $filterCondition);
        $this->session->set('value', $match);
        $this->session->set('andOr', $filterAndOr);
        $this->session->set('edit', $this->request->getParameter('edit'));
        $this->saveParameterForNextPage('filterActionFinish', $filterActionFinish);
        $this->saveParameterForNextPage('filterAction', $filterAction);
        $this->saveParameterForNextPage('destination', $filterActionValue);
      }
      else
      {
        $this->session->set('maxLine', 1);
        $this->session->set('filterTest', array());
        $this->session->set('filterCondition', array());
        $this->session->set('value', array());
        $this->session->set('andOr', array());
      }
    }
    else
    {
      if($this->request->getSubmitParameter('action') == 'chgAdminUser')
      {
        if($this->request->getParameter('adminUser') != '__SELF__')
          $acc = Account::findByPK(array('accountname' => $this->request->getParameter('adminUser')));
        else
          $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        $this->session->set('usedAccount', $acc);
        $this->saveParameterForNextPage('account', $acc);
        $this->saveParameterForNextPage('adminUser', $this->request->getParameter('adminUser'));
      }
      elseif($this->request->getSubmitParameter('addLine') != '')
      {
        if($user->getIsAdmin() != 'f')
          $this->saveParameterForNextPage('adminUser', $acc->getAccountname());
        $this->saveParameterForNextPage('account', $acc);
        
        $f = $c = $v = $a = array();
        for($l = 0; $l < $this->session->get('maxLine'); $l++)
        {
          array_push($f, $this->request->getParameter('filterTest_'.$l));
          array_push($c, $this->request->getParameter('filterCondition_'.$l));
          array_push($v, $this->request->getParameter('value_'.$l));
          array_push($a, $this->request->getParameter('andOr_'.$l));
        }
        $this->session->set('maxLine', $this->session->get('maxLine') + 1);
        $this->session->set('filterTest', $f);
        $this->session->set('filterCondition', $c);
        $this->session->set('value', $v);
        $this->session->set('andOr', $a);
      }
      elseif($this->request->getSubmitParameter('delLine') != '')
      {
        if($user->getIsAdmin() != 'f')
          $this->saveParameterForNextPage('adminUser', $acc->getAccountname());
        $this->saveParameterForNextPage('account', $acc);

        $f = $c = $v = $a = array();
        $f1 = $this->session->get('filterTest');
        $c1 = $this->session->get('filterCondition');
        $v1 = $this->session->get('value');
        $a1 = $this->session->get('andOr');
        for($l = 0; $l < $this->session->get('maxLine'); $l++)
        {
          if($l != $this->request->getParameter('delLineNo'))
          {
            array_push($f, $f1[$l]);
            array_push($c, $c1[$l]);
            array_push($v, $v1[$l]);
            array_push($a, $a1[$l]);
          }
        }
        $this->session->set('maxLine', $this->session->get('maxLine') - 1);
        $this->session->set('filterTest', $f);
        $this->session->set('filterCondition', $c);
        $this->session->set('value', $v);
        $this->session->set('andOr', $a);
      }
      elseif($this->request->getSubmitParameter('cancel') != '')
      {
        if($user->getIsAdmin() != 'f')
          $this->saveParameterForNextPage('adminUser', $acc->getAccountname());
        $this->saveParameterForNextPage('account', $acc);

        $this->session->delete('filters');
        $this->session->set('edit', NULL);
        $this->session->set('maxLine', 1);
        $this->session->set('filterTest', array());
        $this->session->set('filterCondition', array());
        $this->session->set('value', array());
        $this->session->set('andOr', array());
      }
      elseif($this->request->getSubmitParameter('addFilter') != '')
      {
        $filter = "if\n";
        for($l = 0; $l < $this->session->get('maxLine'); $l++)
        {
          $filter .= ' '.trim($this->request->getParameter('filterTest_'.$l)).' ';
          $filter .= $this->request->getParameter('filterCondition_'.$l).' ';
          $filter .= '"'.$this->request->getParameter('value_'.$l).'"';
          if($this->request->getParameter('andOr_'.$l) != '')
            $filter .= "\n".$this->request->getParameter('andOr_'.$l)."\n";
        }
        list($fAction, $fCheck, $fFinish) = Account::$filtersActions[$this->request->getParameter('filterAction')];
        if($fAction == 'save')
          $filterActionValue = sprintf('%%%%FINALDEST%%%%/.%s', $this->request->getParameter('destination'));
        else
          $filterActionValue = $this->request->getParameter('destination');
        $finish = '';
        if($this->request->getParameter('filterActionFinish') == 't' && $fFinish == 1)
          $finish = "\n seen finish";
        $filter .= "\n then\n $fAction $filterActionValue$finish\nendif";

        $filters = $this->session->get('filters');
        $newFilters = array();
        if($this->session->get('edit') === NULL)
        {
          array_push($filters, $filter);
          for($l = 0; $l < count($filters); $l++)
            array_push($newFilters, $filters[$l]);
          $this->saveParameterForNextPage('popupMessage', _('Der Filter wurde hinzugef&uuml;t und ist sofort wirksam'));
        }
        else
        {
          for($l = 0; $l < count($filters); $l++)
          {
            if($l != $this->session->get('edit'))
              array_push($newFilters, $filters[$l]);
            else
              array_push($newFilters, $filter);
          }
          $this->saveParameterForNextPage('popupMessage', _('Der Filter wurde ge&auml;ndert und ist sofort wirksam'));
        }
        $filterStr = str_replace(' endif', 'endif', join("\n", $newFilters));
        $acc->setFilters($filterStr);
        $acc->save();
        $this->session->delete('filters');
        $this->session->set('edit', NULL);
        $this->session->set('maxLine', 1);
        $this->session->set('filterTest', array());
        $this->session->set('filterCondition', array());
        $this->session->set('value', array());
        $this->session->set('andOr', array());
        if($user->getIsAdmin() != 'f')
          $this->saveParameterForNextPage('adminUser', $acc->getAccountname());
        $this->saveParameterForNextPage('account', $acc);
      }
    }
    $this->saveParameterForNextPage('account', $acc);
    $this->session->set('filters', $acc->getParsedFilters());

    return $response;
  }

  /**
   * Called if user data are not valid (check with validate)
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function manageInvalidData()
  {
    $user = self::getLoggedUser();
    $response = new myMVC_HttpResponse();
    $response->setNext('ManageFilterView');
    $this->saveParameterForNextPage('error', true);

    $this->session->useNamespace('manageFilter');
    $acc = $this->session->get('usedAccount');
    $f = $c = $v = $a = array();
    for($l = 0; $l < $this->session->get('maxLine'); $l++)
    {
      array_push($f, $this->request->getParameter('filterTest_'.$l));
      array_push($c, $this->request->getParameter('filterCondition_'.$l));
      array_push($v, $this->request->getParameter('value_'.$l));
      array_push($a, $this->request->getParameter('andOr_'.$l));
    }
    $this->session->set('filterTest', $f);
    $this->session->set('filterCondition', $c);
    $this->session->set('value', $v);
    $this->session->set('andOr', $a);

    return $response;
  }
}
?>
