<?php
class ManageListAction extends GUIAction
{
  public function __construct($request, $session)
  {
    parent::__construct($request, $session);
    $this->setCheckFunction('POST', 'validatePost');
    $this->setCheckFunction('GET', 'validateGet');
    $this->setCheckFunction('AJAX', 'validateAjax');
  }

  /**
   * Returns the CSS class of this action
   * 
   * @return string                     The CSS class
   */
  public static function getCSSClass()
  {
    return 'general';
  }

  /**
   * Returns the token used to identify the current Action
   * 
   * @return string                 The token
   */
  public static function getActionToken()
  {
    return 'manageList';
  }

  /**
   * Returns whether this action should appear in the menu (created by controller)
   * 
   * @return boolean                true if the action should appear in menu
   */
  public static function isMenuAction()
  {
    return true;
  }

  /**
   * Returns the position of this action in the menu (created by controller)
   * Only necessary if isMenuAction() returns true
   * 
   * @return int                        The position in the menu (lower is "upper")
   */
  public static function getActionMenuPosition()
  {
    return 40;
  }

  /**
   * Returns the string to be displayed in the menu to identify the current action.
   * Only necessary if isMenuAction() returns true
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuName()
  {
    return _("Black-/Whitelisten");
  }

  /**
   * Gibt den Name des Bildes für dieses Menüpunkt zurücl
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuImage()
  {
    return '';
  }

  /**
   * Returns whether the current user is allowed to call the current Action
   * 
   * @return boolean                true if the user may call this Action, false otherwise
   */
  public static function hasUserRightForAction()
  {
    $user = self::getLoggedUser();
    if($user !== NULL)
      return true;
    return false;
  }

  /**
   * Validate the data from user, coming as GET
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validateGet()
  {
    $this->session->useNamespace('manageList');
    $acc = $this->session->get('usedAccount');

    if($this->request->getSubmitParameter('delete') != '')
    {
      $definitions = array(
          'delete' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false,
                                        'comparedb' => array('query' => sprintf("SELECT COUNT(*) FROM userpref WHERE prefid = {checkValue} AND accountname = '%s'",
                                                                            $acc->getAccountname()),
                                            'compareType' => '==', 'compareValue' => 1)),
      );
    }
    $names = array(
          'delete' => _('Einstellung zu l&ouml;schen'),
    );
    $errorMsgs = array(
          'delete' => array(myMVC_BaseAction::$ERROR_COMPAREDB => _('Die Einstellung existiert nicht'),
                            myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                            myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
    );

    if(!is_array($definitions))
      return true;
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Validate the data from user, coming as POST
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validatePost()
  {
    $user = self::getLoggedUser();
    $users = array('__SELF__');
    foreach($user->getUsersForAdmin() as $u)
      array_push($users, $u->getAccountname());

    if($this->request->getSubmitParameter('action') == 'chgAdminUser')
    {
      $definitions = array(
          'adminUser' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $users),
      );
    }
    else
    {
      $definitions = array(
          'newType'   => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => array_keys(Userpref::getPreferenceTypes())),
          'newValue'  => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'maxlen' => 255),
      );
      if($user->getIsAdmin() != 'f')
      {
        $users = array('__SELF__');
        foreach($user->getUsersForAdmin() as $u)
          array_push($users, $u->getAccountname());
        $definitions['adminUser'] =  array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $users);
      }
    }
    $names = array(
        'newType'   => _('Typ'),
        'newValue'  => _('Wert'),
        'adminUser' => _('Administrierter Nutzer'),
    );
    $errorMsgs = array(
        'newType'   => array(myMVC_BaseAction::$ERROR_RANGE     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                             myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                             myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'newValue'  => array(myMVC_BaseAction::$ERROR_MAXLEN    => sprintf(_("%s ist zu lang"), '{field}'),
                             myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                             myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'adminUser' => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                             myMVC_BaseAction::$ERROR_RANGE         => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                             myMVC_BaseAction::$ERROR_NOTSTRING     => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
    );
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Validate the data from user, coming as AJAX.
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validateAjax()
  {
    $definitions = array(
        'search'     => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'minlen' => 2, 'maxlen' => 255, 'empty' => true),
    );
    $names = array(
        'search'     => _('Suchkriterium'),
    );
    $errorMsgs = array(
        'search'     => array(myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                              myMVC_BaseAction::$ERROR_MINLEN    => sprintf(_("%s muss zwischen 3 und 255 Zeichen lang sein"), '{field}'),
                              myMVC_BaseAction::$ERROR_MAXLEN    => sprintf(_("%s muss zwischen 3 und 255 Zeichen lang sein"), '{field}')),
    );
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Perform the Action
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function perform()
  {
    global $config;

    $response = new myMVC_HttpResponse();

    $response->setNext('ManageListView');
    $this->session->useNamespace('manageList');
    $user = self::getLoggedUser();

    if($this->request->getRequestMethod() == 'GET')
    {
      if($this->isCalledFromMenu())
      {
        $this->session->emptyNamespace();
        $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        $this->session->set('usedAccount', $acc);
      }
      else
        $acc = $this->session->get('usedAccount');

      if($this->request->getSubmitParameter('delete') != '')
      {
        $pref = Userpref::findByPK(array('prefid' => $this->request->getParameter('delete')));
        $pref->delete();
        $this->saveParameterForNextPage('popupMessage', _('Die Einstellung wurde erfolgreich gel&ouml;scht'));
        if($user->getIsAdmin() != 'f')
          $this->saveParameterForNextPage('adminUser', $acc->getAccountname());
      }
    }
    else
    {
      if($this->request->getSubmitParameter('action') == 'chgAdminUser')
      {
        if($this->request->getParameter('adminUser') != '__SELF__')
          $acc = Account::findByPK(array('accountname' => $this->request->getParameter('adminUser')));
        else
          $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        $this->session->set('usedAccount', $acc);
        $this->saveParameterForNextPage('account', $acc);
        $this->saveParameterForNextPage('adminUser', $this->request->getParameter('adminUser'));
      }
      else
      {
        $acc = $this->session->get('usedAccount');
        $pref = new Userpref();
        $pref->setPrefid(NULL);
        $pref->setAccountname($acc->getAccountname());
        $pref->setPreference($this->request->getParameter('newType'));
        $pref->setValue($this->request->getParameter('newValue'));
        $pref->save();
        $this->saveParameterForNextPage('popupMessage', _('Die Einstellung wurde erfolgreich gespeichert'));
        if($user->getIsAdmin() != 'f')
          $this->saveParameterForNextPage('adminUser', $this->request->getParameter('adminUser'));
        else
          $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        $this->saveParameterForNextPage('account', $acc);
      }
    }
    $this->saveParameterForNextPage('lists', Userpref::callFinder('findByAccountname', array('accountname' => $acc->getAccountname()), Userpref::getSortString()));

    return $response;
  }

  /**
   * Called if user data are not valid (check with validate)
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function manageInvalidData()
  {
    $user = self::getLoggedUser();
    $response = new myMVC_HttpResponse();
    $response->setNext('ManageListView');
    $this->saveParameterForNextPage('error', true);
    $this->session->useNamespace('manageList');
    $acc = $this->session->get('usedAccount');
    $this->saveParameterForNextPage('lists', Userpref::callFinder('findByAccountname', array('accountname' => $acc->getAccountname()), Userpref::getSortString()));

    return $response;
  }

  /**
   * Perform the AJAX-Action
   *
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function performAjax()
  {
    global $controller;

    $this->session->useNamespace('manageList');
    $acc = $this->session->get('usedAccount');

    $this->session->useNamespace('main');
    header('Content-Type: application/json');
    $user = self::getLoggedUser();

    if($this->request->getParameter('search') != '')
      $lists = Userpref::callFinder('findByAccountnameAndValue', array('accountname' => $acc->getAccountname(), 'value' => $this->request->getParameter('search')), Userpref::getSortString());
    else
      $lists = Userpref::callFinder('findByAccountname', array('accountname' => $acc->getAccountname()), Userpref::getSortString());

    $renderEngine = new myMVC_SmartyEngine(dirname(__FILE__).'/../pages/templates',
                                           dirname(__FILE__).'/../pages/templates_c');
    $view = new SearchView($renderEngine, $this->session, 'searchLists.tpl');
    $view->setObject('lists', $lists);
    $view->setObject('types', Userpref::getPreferenceTypes());
    $controller->setCurrentLanguage();
    if($view->preRender())
      $html = $view->fetch();
    else
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'SearchView::preRender() returned false. Unable to display page');
      throw new myMVC_MVCException($msg);
    }

    print json_encode(array('result' => 'OK', 'html' => $html));
    exit;
  }

  /**
   * Called if user data are not valid (check with validate)
   *
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function manageInvalidDataAjax()
  {
    header('Content-Type: application/json');
    $errors = $this->session->getFromNamespace('FORMCHECK', 'ERRORS');
    $ret = array('result' => 'ERROR', 'error' => $errors['search']);
    print json_encode($ret);
    exit;
  }
}
?>
